<?php
/* -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
# ***** BEGIN LICENSE BLOCK *****
# This file is part of Plume CMS, a website management application.
# Copyright (C) 2001-2005 Loic d'Anterroches and contributors.
#
# Plume CMS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Plume CMS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# ***** END LICENSE BLOCK ***** */

require_once dirname(__FILE__).'/class.l10n.php';

class Search extends CError
{

    var $websiteId = ''; //Website id
    var $con = null; //DB connection

    /**
    Constructor

    @param object DB reference
    @param string Website id
    */
    function Search(&$db, $websiteId='')
    {
        $this->con = $db;
        $this->websiteId = $websiteId;
        return true;
    }

    /**
    Index a text string.

    @param string The text string to index
    @param string The identifier of the string (resource id for example)
    @param string The type of string
    @return bool Succes or not, if error an error message is set
    */
    function index($string, $id, $type = 'html')
    {
        if ('html' == $type) {
            //To have an accurate number of occurences
            $string = $this->balance_string($string);
        }
        $string = $this->clean_string($string);
        $words  = $this->explode_in_words($string);
        return $this->save_words($id, $words);
    }


    /**
    Clean string.

    Clean a string to be used for the indexation. For the moment consider the string as having
    only text (or wiki) but not HTML.

    @param string String to clean
    @return string Cleaned string
    */
    function clean_string($string)
    {
        include_once dirname(__FILE__).'/lib.text.php';
        $string = text::removeEntities($string);
        if (strtolower(config::f('encoding') == 'utf-8')) {
            $string = utf8_decode($string);
        }
        $string = strtr($string,"","AAAAAAaaaaaaOOOOOOooooooEEEEeeeeIIIIiiiiUUUUuuuuyNn");
        $string = preg_replace('/[_]{2,}/', ' ', $string);
        $string = preg_replace('/[^a-zA-Z0-9_\-\s]/', ' ', $string);
        return $string;
    }

    /**
     * Balance HTML string.
     *
     * Balance an HTML string by identification of words in headlines. Return a
     * string with more occurences of the important words.
     *
     * @param string String to balance
     * @return string Balanced string
     */
    function balance_string($string)
    {
        return $string;
    }

    /**
     * Explode in words.
     *
     * Explode in words a strings and give the number of occurence 
     * of each word.
     *
     * @param string String to be exploded
     * @param bool Remove the small words and numerical only (true)
     * @return array Associative array, key: word, value: occurence
     */
    function explode_in_words($string, $remove=true)
    {
        $words = array();
        $twords = preg_split('/\s+/', $string);
        foreach ($twords as $word) {
            $word = trim(strtolower($word));
            if ($remove && preg_match('/^[0-9]+$/', $word)) {
                $word = '';
            }
            if (!empty($word) && (!$remove or strlen($word) > 2)) {
                if (!empty($words[$word])) {
                    $words[$word]++;
                } else {
                    $words[$word] = 1;
                }
            }
        }
        return $words;
    }

    /**
     * Save the indexed resource.
     *
     * @param string Resource id
     * @param array Words and occurences
     * @return bool Success
     */
    function save_words($id, $words)
    {
        //save the words in the db
        return $this->db_save_words($id, $words);
    }

    /**
     * Add the words to the word table. The words are supposed to be clean.
     * Update the occurence table
     *
     * @param string Resource id
     * @param array words
     * @return bool Success
     */
    function db_save_words($id, $words)
    {
        include_once dirname(__FILE__).'/lib.utils.php';

        $reqExists = 'SELECT * FROM '.$this->con->pfx.'searchwords 
                      WHERE word=\'%s\'';
        $reqIns    = 'INSERT INTO '.$this->con->pfx.'searchwords SET 
                      word=\'%s\'';
        $reqInsOcc = 'INSERT INTO '.$this->con->pfx.'searchocc SET
                      word_id=\'%s\', 
                      resource_id=\''.$this->con->esc($id).'\',
                      website_id=\''.$this->con->esc($this->websiteId).'\',
                      occ=\'%s\', pondocc=\'%s\'';

        // Remove the previous indexation of the resource
        if (false === $this->remove_from_index($id)) {
            return false;
        }           

        // Add the new words (in the word table and in the occurence table)
        // Number of indexed words in the string
        reset($words);
        $total = 0.0;
        foreach ($words as $word => $occ) {
            $total += (float) $occ;
        }
        reset($words);
        foreach ($words as $word => $occ) {
            $req = sprintf($reqExists, $this->con->escapeStr($word));
            if (($rs = $this->con->select($req)) === false) {
                $this->setError('MySQL: '.$this->con->error(), 500);
                return false;
            }
            if ($rs->nbRow() == 0) {
                // need to add the word to the word table
                $req = sprintf($reqIns, $this->con->escapeStr($word));
                if (!$this->con->execute($req)) {
                    $this->setError('MySQL: '.$this->con->error(), 500);
                    return false;
                }
                if (false == ($wid = $this->con->getLastID())) {
                    $this->setError('MySQL: '.$this->con->error(), 500);
                    return false;
                }
            } else {
                $wid = $rs->f('word_id');
            }
            // Add the word to the occurence table
            $req = sprintf($reqInsOcc, $wid, $this->con->escapeStr($occ), 
                           (float) $occ/$total*100.0);
            if (!$this->con->execute($req)) {
                $this->setError('MySQL: '.$this->con->error(), 500);
                return false;
            }
        }

        // Update the stats table
        $reqSel = 'SELECT * FROM '.$this->con->pfx.'search 
                   WHERE resource_id=\''.$this->con->escapeStr($id).'\'';
        $reqIns = 'INSERT INTO '.$this->con->pfx.'search SET 
                   resource_id=\''.$this->con->escapeStr($id).'\',
                   website_id=\''.$this->con->escapeStr($this->websiteId).'\', 
                   lastindex=\''.date::stamp().'\',
                   nbindex=\'1\'';
        $reqUpd = 'UPDATE '.$this->con->pfx.'search SET 
                   lastindex=\''.date::stamp().'\', nbindex=nbindex+1
                   WHERE resource_id=\''.$this->con->escapeStr($id).'\'';

        if (($rs = $this->con->select($reqSel)) === false) {
            $this->setError('MySQL: '.$this->con->error(), 500);
            return false;
        }
        if ($rs->nbRow() == 0) {
            $req = $reqIns;
        } else {
            $req = $reqUpd;
        }
        if (!$this->con->execute($req)) {
            $this->setError('MySQL: '.$this->con->error(), 500);
            return false;
        }
        return true;
    }
    
    /**
     * Remove a resource from the index. 
     *    
     * @param string Resource id
     * @return bool Succes
     */
    function remove_from_index($id)
    {
        // Remove the previous indexation of the resource
        $req = 'DELETE FROM '.$this->con->pfx.'searchocc WHERE resource_id=\''
            .$this->con->escapeStr($id).'\'';
        if (!$this->con->execute($req)) {
            $this->setError('MySQL: '.$this->con->error(), 500);
            return false;
        }   
        return true;
    }
    
    /**
     * Clean index. 
     *
     * Clean the index of the words that are not used anymore.
     * This a SQL intensive function. 
     *    
     * @return int Number of word removed or false
     */
    function clean_index()
    {
        //Get all the word ids.
        $i = 0;
        $req = 'SELECT word_id FROM '.$this->con->pfx.'searchwords';
        if (($rs = $this->con->select($req)) === false) {
            $this->setError('MySQL: '.$this->con->error(), 500);
            return false;
        }
        while (!$rs->EOF()) {
            $req =  'SELECT resource_id FROM '.$this->con->pfx.'searchocc ';
            $req .= 'WHERE word_id=\''.$this->con->escapeStr($rs->f('word_id'))
                .'\'';
            if (($rsocc = $this->con->select($req)) === false) {
                $this->setError('MySQL: '.$this->con->error(), 500);
                return false;
            }
            if ($rsocc->nbRow() == 0) {
                // the word is not used, delete it
                $req =  'DELETE FROM '.$this->con->pfx.'searchwords ';
                $req .= 'WHERE word_id=\''
                    .$this->con->escapeStr($rs->f('word_id')).'\'';
                if (!$this->con->execute($req)) {
                    $this->setError('MySQL: '.$this->con->error(), 500);
                    return false;
                }
                $i++;
            }
            $rs->moveNext();
        }
        return $i;
    }
    


    /**
     * Return the basic SQL query with place holders to generate 
     * the complete search query.
     *    
     * @param string Query string sent by the user
     * @return string SQL basic query string
     */
    function create_search_query_string($query)
    {
        $query = $this->clean_string($query);
        $words = $this->explode_in_words($query, false);
        $kw = array();
        reset($words);
        $nbwords = 0;
        foreach ($words as $word => $occ) {
            $kw[] = $this->con->pfx.'searchwords.word LIKE \''.$this->con->escapeStr($word).'\'';
            $nbwords++;
        }
        $orstring = implode(' OR ', $kw);
        if (strlen($orstring) == 0) $orstring = '1=0';
        $sql  = 'SELECT '.$this->con->pfx.'resources.*, '
            .$this->con->pfx.'categories.*, COUNT(*) AS total, '."\n";
        $sql .= 'SUM('.$this->con->pfx.'searchocc.occ) AS score '."\n";
        $sql .= 'FROM '.$this->con->pfx.'searchocc'."\n";
        $sql .= 'LEFT JOIN '.$this->con->pfx.'searchwords ON '
            .$this->con->pfx.'searchwords.word_id='.$this->con->pfx
            .'searchocc.word_id '."\n";
        $sql .= 'LEFT JOIN '.$this->con->pfx.'resources ON '
            .$this->con->pfx.'resources.resource_id='.$this->con->pfx
            .'searchocc.resource_id '."\n";
        $sql .= 'LEFT JOIN '.$this->con->pfx.'categoryasso ON '
            .$this->con->pfx.'categoryasso.identifier='.$this->con->pfx
            .'resources.identifier '."\n";
        $sql .= 'LEFT JOIN '.$this->con->pfx.'categories ON '
            .$this->con->pfx.'categoryasso.category_id='.$this->con->pfx
            .'categories.category_id '."\n";
        $sql .= 'WHERE ('.$orstring.')'."\n";
        $sql .= 'AND '.$this->con->pfx.'categoryasso.categoryasso_type=\''
            .PX_RESOURCE_CATEGORY_MAIN.'\' '."\n";
        $sql .= 'AND '.$this->con->pfx.'resources.website_id=\''
            .$this->websiteId.'\' %s '."\n";
        $sql .= 'GROUP BY '.$this->con->pfx
            .'searchocc.resource_id HAVING total=\''.$nbwords
            .'\' ORDER BY total DESC, score DESC';

        return $sql;
    }
    
    /**
     * Get the list of resources in a website with the last indexation time.
     *    
     * @param string Website to look for, if no set, all
     * @return mixed RecordSet or false if errors
     */
    function get_indexed_resources_stats($website='')
    {
        $website = trim($website);
        $extra = '';
        if (!empty($website)) {
            $extra = 'WHERE '.$this->con->pfx.'resources.website_id=\''
                .$this->con->esc($website).'\''."\n";
        }
        $sql  = 'SELECT * FROM '.$this->con->pfx.'resources '."\n";
        $sql .= 'LEFT JOIN '.$this->con->pfx.'search 
           ON '.$this->con->pfx.'resources.resource_id='
           .$this->con->pfx.'search.resource_id '."\n";
        $sql .= $extra;
        $sql .= 'ORDER BY lastindex ASC';
        if (($rs = $this->con->select($sql)) === false) {
            $this->setError('MySQL: '.$this->con->error(), 500);
            return false;
        }
        return $rs;
    }

    /* ===================================================================== *
     *                                                                       *
     *                Methods for rendering the pages.                       *
     *                                                                       *
     * Note: All standalone methods.                                         *
     * ===================================================================== */

    /**
     * Action to display a category.
     *
     * @param string Server query string
     * @return int Success code
     */
    function action($query)
    {
        // Easy access
        $GLOBALS['_PX_render']['last'] = '';
        $last =& $GLOBALS['_PX_render']['last']; 
        $GLOBALS['_PX_render']['website'] = '';
        $website =& $GLOBALS['_PX_render']['website']; 
        $GLOBALS['_PX_render']['res'] = '';
        $res =& $GLOBALS['_PX_render']['res']; 

        // Parse query string to find the matching article
        $con =& pxDBConnect();
        $s = new Search($con, config::f('website_id'));
        config::setVar('query_string', Search::parseQueryString($query));

        $sql = $s->create_search_query_string(config::f('query_string'));
        $extra = ' AND '.$con->pfx.'resources.publicationdate <= '
            .date::stamp().' AND '.$con->pfx.'resources.enddate >= '
            .date::stamp().' AND '.$con->pfx.'resources.status = '
            .PX_RESOURCE_STATUS_VALIDE;
        $sql = sprintf($sql, $extra);

        if (($res = $con->select($sql, 'ResourceSet')) === false) {
            $GLOBALS['_PX_render']['error']->setError('MySQL: '
                                                      .$con->error(), 500);
            return 404;
        }

        $GLOBALS['_PX_render']['res']->autoSave(array('score', 'total'));
        $GLOBALS['_PX_render']['website'] = FrontEnd::getWebsite();

        header(FrontEnd::getHeader('search.php'));
        // Load the template
        include config::f('manager_path').'/templates/'
            .config::f('theme_id').'/search.php';
        return 200;
    }

    /**
     * Parse query string.
     *
     * @param string Query string
     * @return string Clean query string
     */
    function parseQueryString($query)
    {
        $clean = '';
        if (preg_match('#^/search/(.*)$#i', $query, $match)) {
            $clean = $match[1];
            if (empty($clean)) {
                if (!empty($_GET['q'])) {
                    $clean = $_GET['q'];
                }
            }
            $clean = Search::clean_string($clean);
            $words = Search::explode_in_words($clean, true);
            $clean = '';
            foreach ($words as $k => $v) {
                $clean .= $k.' ';
            }
            $clean = trim($clean);
        } 
        return $clean;
    }


}



?>